import pyarrow.parquet as pq
import pandas as pd
import json
from typing import List, Callable, Iterator, Union, Optional
from sportsdataverse.errors import SeasonNotFoundError
from sportsdataverse.dl_utils import download

def espn_wbb_schedule(dates=None, groups=None, season_type=None, limit=500) -> pd.DataFrame:
    """espn_wbb_schedule - look up the women's college basketball schedule for a given season

    Args:
        dates (int): Used to define different seasons. 2002 is the earliest available season.
        groups (int): Used to define different divisions. 50 is Division I, 51 is Division II/Division III.
        season_type (int): 2 for regular season, 3 for post-season, 4 for off-season.
        limit (int): number of records to return, default: 500.

    Returns:
        pd.DataFrame: Pandas dataframe containing schedule dates for the requested season.
    """
    if dates is None:
        dates = ''
    else:
        dates = '&dates=' + str(dates)
    if groups is None:
        groups = '&groups=50'
    else:
        groups = '&groups=' + str(groups)
    if season_type is None:
        season_type = ''
    else:
        season_type = '&seasontype=' + str(season_type)
    url = "http://site.api.espn.com/apis/site/v2/sports/basketball/womens-college-basketball/scoreboard?limit={}{}{}{}".format(limit, dates, groups, season_type)
    resp = download(url=url)

    ev = pd.DataFrame()
    if resp is not None:
        events_txt = json.loads(resp)

        events = events_txt.get('events')
        for event in events:
            event.get('competitions')[0].get('competitors')[0].get('team').pop('links',None)
            event.get('competitions')[0].get('competitors')[1].get('team').pop('links',None)
            if event.get('competitions')[0].get('competitors')[0].get('homeAway')=='home':
                event['competitions'][0]['home'] = event.get('competitions')[0].get('competitors')[0].get('team')
                event['competitions'][0]['away'] = event.get('competitions')[0].get('competitors')[1].get('team')
            else:
                event['competitions'][0]['away'] = event.get('competitions')[0].get('competitors')[0].get('team')
                event['competitions'][0]['home'] = event.get('competitions')[0].get('competitors')[1].get('team')

            del_keys = ['broadcasts','geoBroadcasts', 'headlines', 'series']
            for k in del_keys:
                event.get('competitions')[0].pop(k, None)
            x = pd.json_normalize(event.get('competitions')[0])
            x['game_id'] = x['id'].astype(int)
            x['season'] = event.get('season').get('year')
            x['season_type'] = event.get('season').get('type')
            ev = ev.append(x)
    ev = pd.DataFrame(ev)
    return ev

def espn_wbb_calendar(season=None) -> pd.DataFrame:
    """espn_wbb_calendar - look up the women's college basketball calendar for a given season

    Args:
        season (int): Used to define different seasons. 2002 is the earliest available season.

    Returns:
        pd.DataFrame: Pandas dataframe containing
        calendar dates for the requested season.

    Raises:
        ValueError: If `season` is less than 2002.
    """
    if int(season) < 2002:
        raise SeasonNotFoundError("season cannot be less than 2002")
    url = "http://site.api.espn.com/apis/site/v2/sports/basketball/womens-college-basketball/scoreboard?dates={}".format(season)
    resp = download(url=url)
    txt = json.loads(resp)['leagues'][0]['calendar']
    datenum = list(map(lambda x: x[:10].replace("-",""),txt))
    date = list(map(lambda x: x[:10],txt))

    year = list(map(lambda x: x[:4],txt))
    month = list(map(lambda x: x[5:7],txt))
    day = list(map(lambda x: x[8:10],txt))

    data = {
        "season": season,
        "datetime" : txt,
        "date" : date,
        "year": year,
        "month": month,
        "day": day,
        "dateURL": datenum
    }
    df = pd.DataFrame(data)
    df['url']="http://site.api.espn.com/apis/site/v2/sports/basketball/womens-college-basketball/scoreboard?dates="
    df['url']= df['url'] + df['dateURL']
    return df




import pandas as pd
import json
from sportsdataverse.dl_utils import download
from urllib.error import URLError, HTTPError, ContentTooShortError

def espn_wbb_teams(groups=None) -> pd.DataFrame:
    """espn_wbb_teams - look up the women's college basketball teams

    Args:
        groups (int): Used to define different divisions. 50 is Division I, 51 is Division II/Division III.

    Returns:
        pd.DataFrame: Pandas dataframe containing teams for the requested league.
    """
    if groups is None:
        groups = '&groups=50'
    else:
        groups = '&groups=' + str(groups)
    ev = pd.DataFrame()
    url = "http://site.api.espn.com/apis/site/v2/sports/basketball/womens-college-basketball/teams?limit=1000{}".format(groups)
    resp = download(url=url)
    if resp is not None:
        events_txt = json.loads(resp)

        teams = events_txt.get('sports')[0].get('leagues')[0].get('teams')
        del_keys = ['record', 'links']
        for team in teams:
            for k in del_keys:
                team.get('team').pop(k, None)
        teams = pd.json_normalize(teams)
    return teams




import pandas as pd
import json
from sportsdataverse.dl_utils import download
from urllib.error import URLError, HTTPError, ContentTooShortError

def espn_cfb_schedule(dates=None, week=None, season_type=None, groups=None, limit=500) -> pd.DataFrame:
    """espn_cfb_schedule - look up the college football schedule for a given season

    Args:
        dates (int): Used to define different seasons. 2002 is the earliest available season.
        week (int): Week of the schedule.
        groups (int): Used to define different divisions. 80 is FBS, 81 is FCS.
        season_type (int): 2 for regular season, 3 for post-season, 4 for off-season.
        limit (int): number of records to return, default: 500.

    Returns:
        pd.DataFrame: Pandas dataframe containing schedule dates for the requested season.
    """
    if week is None:
        week = ''
    else:
        week = '&week=' + str(week)
    if dates is None:
        dates = ''
    else:
        dates = '&dates=' + str(dates)
    if season_type is None:
        season_type = ''
    else:
        season_type = '&seasontype=' + str(season_type)
    if groups is None:
        groups = '&groups=80'
    else:
        groups = '&groups=' + str(groups)

    url = "http://site.api.espn.com/apis/site/v2/sports/football/college-football/scoreboard?limit={}{}{}{}{}".format(limit, groups,dates,week,season_type)
    resp = download(url=url)

    ev = pd.DataFrame()
    if resp is not None:
        events_txt = json.loads(resp)

        events = events_txt.get('events')
        for event in events:
            event.get('competitions')[0].get('competitors')[0].get('team').pop('links',None)
            event.get('competitions')[0].get('competitors')[1].get('team').pop('links',None)
            if event.get('competitions')[0].get('competitors')[0].get('homeAway')=='home':
                event['competitions'][0]['home'] = event.get('competitions')[0].get('competitors')[0].get('team')
                event['competitions'][0]['away'] = event.get('competitions')[0].get('competitors')[1].get('team')
            else:
                event['competitions'][0]['away'] = event.get('competitions')[0].get('competitors')[0].get('team')
                event['competitions'][0]['home'] = event.get('competitions')[0].get('competitors')[1].get('team')

            del_keys = ['broadcasts','geoBroadcasts', 'headlines', 'series']
            for k in del_keys:
                event.get('competitions')[0].pop(k, None)
            x = pd.json_normalize(event.get('competitions')[0])
            x['game_id'] = x['id'].astype(int)
            x['season'] = event.get('season').get('year')
            x['season_type'] = event.get('season').get('type')
            ev = ev.append(x)
    ev = pd.DataFrame(ev)
    return ev



def espn_cfb_calendar(season=None, groups=None) -> pd.DataFrame:
    """espn_cfb_calendar - look up the men's college football calendar for a given season

    Args:
        season (int): Used to define different seasons. 2002 is the earliest available season.
        groups (int): Used to define different divisions. 80 is FBS, 81 is FCS.

    Returns:
        pd.DataFrame: Pandas dataframe containing calendar dates for the requested season.

    Raises:
        ValueError: If `season` is less than 2002.
    """
    if season is None:
        season = ''
    else:
        season = '&dates=' + str(season)
    if groups is None:
        groups = '&groups=80'
    else:
        groups = '&groups=' + str(groups)
    url = "http://site.api.espn.com/apis/site/v2/sports/football/college-football/scoreboard?{}{}".format(season, groups)
    resp = download(url=url)
    txt = json.loads(resp)['leagues'][0]['calendar']
    full_schedule = pd.DataFrame()
    for i in range(len(txt)):
        reg = pd.DataFrame(txt[i]['entries'])
        full_schedule = pd.concat([full_schedule,reg], ignore_index=True)
    full_schedule['season']=season
    return full_schedule



ep_class_to_score_mapping = {
    0: 7,
    1: -7,
    2: 3,
    3: -3,
    4: 2,
    5: -2,
    6: 0
}

wp_start_touchback_columns = [
    "start.pos_team_receives_2H_kickoff",
    "start.spread_time",
    "start.TimeSecsRem",
    "start.adj_TimeSecsRem",
    "start.ExpScoreDiff_Time_Ratio_touchback",
    "pos_score_diff_start",
    "start.down",
    "start.distance",
    "start.yardsToEndzone.touchback",
    "start.is_home",
    "start.posTeamTimeouts",
    "start.defPosTeamTimeouts",
    "period"
]
wp_start_columns = [
    "start.pos_team_receives_2H_kickoff",
    "start.spread_time",
    "start.TimeSecsRem",
    "start.adj_TimeSecsRem",
    "start.ExpScoreDiff_Time_Ratio",
    "pos_score_diff_start",
    "start.down",
    "start.distance",
    "start.yardsToEndzone",
    "start.is_home",
    "start.posTeamTimeouts",
    "start.defPosTeamTimeouts",
    "period"
]
wp_end_columns = [
    "end.pos_team_receives_2H_kickoff",
    "end.spread_time",
    "end.TimeSecsRem",
    "end.adj_TimeSecsRem",
    "end.ExpScoreDiff_Time_Ratio",
    "end.pos_score_diff",
    "end.down",
    "end.distance",
    "end.yardsToEndzone",
    "end.is_home",
    "end.posTeamTimeouts",
    "end.defPosTeamTimeouts",
    "period"
]

ep_start_touchback_columns = [
    "start.TimeSecsRem",
    "start.yardsToEndzone.touchback",
    "distance",
    "down_1",
    "down_2",
    "down_3",
    "down_4",
    "pos_score_diff_start"
]
ep_start_columns = [
    "start.TimeSecsRem",
    "start.yardsToEndzone",
    "start.distance",
    "down_1",
    "down_2",
    "down_3",
    "down_4",
    "pos_score_diff_start"
]
ep_end_columns = [
    "end.TimeSecsRem",
    "end.yardsToEndzone",
    "end.distance",
    "down_1_end",
    "down_2_end",
    "down_3_end",
    "down_4_end",
    "pos_score_diff_end"
]

ep_final_names = [
    "TimeSecsRem",
    "yards_to_goal",
    "distance",
    "down_1",
    "down_2",
    "down_3",
    "down_4",
    "pos_score_diff_start"
]
wp_final_names = [
    "pos_team_receives_2H_kickoff",
    "spread_time",
    "TimeSecsRem",
    "adj_TimeSecsRem",
    "ExpScoreDiff_Time_Ratio",
    "pos_score_diff_start",
    "down",
    "distance",
    "yards_to_goal",
    "is_home",
    "pos_team_timeouts_rem_before",
    "def_pos_team_timeouts_rem_before",
    "period"
]

    #-------Play type vectors-------------
scores_vec = [
    "Blocked Punt Touchdown",
    "Blocked Punt (Safety)",
    "Punt (Safety)",
    "Blocked Field Goal Touchdown",
    "Missed Field Goal Return Touchdown",
    "Fumble Recovery (Opponent) Touchdown",
    "Fumble Return Touchdown",
    "Interception Return Touchdown",
    "Pass Interception Return Touchdown",
    "Punt Touchdown",
    "Punt Return Touchdown",
    "Sack Touchdown",
    "Uncategorized Touchdown",
    "Defensive 2pt Conversion",
    "Uncategorized",
    "Two Point Rush",
    "Safety",
    "Penalty (Safety)",
    "Punt Team Fumble Recovery Touchdown",
    "Kickoff Team Fumble Recovery Touchdown",
    "Kickoff (Safety)",
    "Passing Touchdown",
    "Rushing Touchdown",
    "Field Goal Good",
    "Pass Reception Touchdown",
    "Fumble Recovery (Own) Touchdown"
]
defense_score_vec = [
    "Blocked Punt Touchdown",
    "Blocked Field Goal Touchdown",
    "Missed Field Goal Return Touchdown",
    "Punt Return Touchdown",
    "Fumble Recovery (Opponent) Touchdown",
    "Fumble Return Touchdown",
    "Kickoff Touchdown", #<--- Kickoff Team recovers the return team fumble and scores
    "Defensive 2pt Conversion",
    "Safety",
    "Sack Touchdown",
    "Interception Return Touchdown",
    "Pass Interception Return Touchdown",
    "Uncategorized Touchdown"
]
turnover_vec = [
    "Blocked Field Goal",
    "Blocked Field Goal Touchdown",
    "Blocked Punt",
    "Blocked Punt Touchdown",
    "Field Goal Missed",
    "Missed Field Goal Return",
    "Missed Field Goal Return Touchdown",
    "Fumble Recovery (Opponent)",
    "Fumble Recovery (Opponent) Touchdown",
    "Fumble Return Touchdown",
    "Defensive 2pt Conversion",
    "Interception",
    "Interception Return",
    "Interception Return Touchdown",
    "Pass Interception Return",
    "Pass Interception Return Touchdown",
    "Kickoff Team Fumble Recovery",
    "Kickoff Team Fumble Recovery Touchdown",
    "Punt Touchdown",
    "Punt Return Touchdown",
    "Sack Touchdown",
    "Uncategorized Touchdown"
]
normalplay = [
    "Rush",
    "Pass",
    "Pass Reception",
    "Pass Incompletion",
    "Pass Completion",
    "Sack",
    "Fumble Recovery (Own)"
]
penalty = [
    'Penalty',
    'Penalty (Kickoff)',
    'Penalty (Safety)'
]
offense_score_vec = [
    "Passing Touchdown",
    "Rushing Touchdown",
    "Field Goal Good",
    "Pass Reception Touchdown",
    "Fumble Recovery (Own) Touchdown",
    "Punt Touchdown", #<--- Punting Team recovers the return team fumble and scores
    "Punt Team Fumble Recovery Touchdown",
    "Kickoff Return Touchdown",
    "Kickoff Team Fumble Recovery Touchdown"
]
punt_vec = [
    "Blocked Punt",
    "Blocked Punt Touchdown",
    "Blocked Punt (Safety)",
    "Punt (Safety)",
    "Punt",
    "Punt Return",
    "Punt Touchdown",
    "Punt Team Fumble Recovery",
    "Punt Team Fumble Recovery Touchdown",
    "Punt Return Touchdown"
]
kickoff_vec = [
    "Kickoff",
    "Kickoff Return (Offense)",
    "Kickoff Return Touchdown",
    "Kickoff Touchdown",
    "Kickoff Team Fumble Recovery",
    "Kickoff Team Fumble Recovery Touchdown",
    "Kickoff (Safety)",
    "Penalty (Kickoff)"
]
int_vec = [
    "Interception",
    "Interception Return",
    "Interception Return Touchdown",
    "Pass Interception",
    "Pass Interception Return",
    "Pass Interception Return Touchdown"
]
end_change_vec = [
    "Blocked Field Goal",
    "Blocked Field Goal Touchdown",
    "Field Goal Missed",
    "Missed Field Goal Return",
    "Missed Field Goal Return Touchdown",
    "Blocked Punt",
    "Blocked Punt Touchdown",
    "Punt",
    "Punt Return",
    "Punt Touchdown",
    "Punt Return Touchdown",
    "Kickoff Team Fumble Recovery",
    "Kickoff Team Fumble Recovery Touchdown",
    "Fumble Recovery (Opponent)",
    "Fumble Recovery (Opponent) Touchdown",
    "Fumble Return Touchdown",
    "Sack Touchdown",
    "Defensive 2pt Conversion",
    "Interception",
    "Interception Return",
    "Interception Return Touchdown",
    "Pass Interception Return",
    "Pass Interception Return Touchdown",
    "Uncategorized Touchdown"
]
kickoff_turnovers = [
    "Kickoff Team Fumble Recovery",
    "Kickoff Team Fumble Recovery Touchdown"
]

qbr_vars = [
    "qbr_epa",
    "sack_epa",
    "pass_epa",
    "rush_epa",
    "pen_epa",
    "spread"
]


import pyarrow.parquet as pq
import pandas as pd
import numpy as np
import json
from typing import List, Callable, Iterator, Union, Optional
from sportsdataverse.errors import SeasonNotFoundError
from sportsdataverse.dl_utils import download

def espn_mbb_schedule(dates=None, groups=None, season_type=None, limit=500) -> pd.DataFrame:
    """espn_mbb_schedule - look up the men's college basketball scheduler for a given season

    Args:
        dates (int): Used to define different seasons. 2002 is the earliest available season.
        groups (int): Used to define different divisions. 50 is Division I, 51 is Division II/Division III.
        season_type (int): 2 for regular season, 3 for post-season, 4 for off-season.
        limit (int): number of records to return, default: 500.
    Returns:
        pd.DataFrame: Pandas dataframe containing schedule dates for the requested season.
    """
    if dates is None:
        dates = ''
    else:
        dates = '&dates=' + str(dates)
    if groups is None:
        groups = '&groups=50'
    else:
        groups = '&groups=' + str(groups)
    if season_type is None:
        season_type = ''
    else:
        season_type = '&seasontype=' + str(season_type)
    url = "http://site.api.espn.com/apis/site/v2/sports/basketball/mens-college-basketball/scoreboard?limit={}{}{}{}".format(limit, dates, groups, season_type)
    resp = download(url=url)

    ev = pd.DataFrame()
    if resp is not None:
        events_txt = json.loads(resp)

        events = events_txt.get('events')
        for event in events:
            event.get('competitions')[0].get('competitors')[0].get('team').pop('links',None)
            event.get('competitions')[0].get('competitors')[1].get('team').pop('links',None)
            if event.get('competitions')[0].get('competitors')[0].get('homeAway')=='home':
                event['competitions'][0]['home'] = event.get('competitions')[0].get('competitors')[0].get('team')
                event['competitions'][0]['home']['score'] = event.get('competitions')[0].get('competitors')[0].get('score')
                event['competitions'][0]['home']['winner'] = event.get('competitions')[0].get('competitors')[0].get('winner')
                event['competitions'][0]['away'] = event.get('competitions')[0].get('competitors')[1].get('team')
                event['competitions'][0]['away']['score'] = event.get('competitions')[0].get('competitors')[1].get('score')
                event['competitions'][0]['away']['winner'] = event.get('competitions')[0].get('competitors')[1].get('winner')
            else:
                event['competitions'][0]['away'] = event.get('competitions')[0].get('competitors')[0].get('team')
                event['competitions'][0]['away']['score'] = event.get('competitions')[0].get('competitors')[0].get('score')
                event['competitions'][0]['away']['winner'] = event.get('competitions')[0].get('competitors')[0].get('winner')
                event['competitions'][0]['home'] = event.get('competitions')[0].get('competitors')[1].get('team')
                event['competitions'][0]['home']['score'] = event.get('competitions')[0].get('competitors')[1].get('score')
                event['competitions'][0]['home']['winner'] = event.get('competitions')[0].get('competitors')[1].get('winner')

            del_keys = ['broadcasts','geoBroadcasts', 'headlines', 'series', 'situation', 'tickets', 'odds']
            for k in del_keys:
                event.get('competitions')[0].pop(k, None)
            if len(event.get('competitions')[0]['notes'])>0:
                event.get('competitions')[0]['notes_type'] = event.get('competitions')[0]['notes'][0].get("type")
                event.get('competitions')[0]['notes_headline'] = event.get('competitions')[0]['notes'][0].get("headline").replace('"','')
            else:
                event.get('competitions')[0]['notes_type'] = ''
                event.get('competitions')[0]['notes_headline'] = ''
            event.get('competitions')[0].pop('notes', None)
            x = pd.json_normalize(event.get('competitions')[0])
            x['game_id'] = x['id'].astype(int)
            x['season'] = event.get('season').get('year')
            x['season_type'] = event.get('season').get('type')
            ev = ev.append(x)
    ev = pd.DataFrame(ev)
    # ev = ev.astype({
    #     'id': int,
    #     'uid': str,
    #     'date': str,
    #     'notes_type': str,
    #     'notes_headline': str,
    #     'type.id': int,
    #     'type.abbreviation': str,
    #     'venue.id': int,
    #     'venue.fullName': str,
    #     'venue.address.city': str,
    #     'venue.address.state': str,
    #     'venue.capacity': int,
    #     'venue.indoor': bool,
    #     'status.clock': str,
    #     'status.displayClock': str,
    #     'status.period ': int,
    #     'status.type.id': int,
    #     'status.type.name': str,
    #     'status.type.state': str,
    #     'status.type.completed': bool,
    #     'status.type.description': str,
    #     'status.type.detail': str,
    #     'status.type.shortDetail': str,
    #     'format.regulation.periods': int,
    #     'home.id': int,
    #     'home.uid': str,
    #     'home.location': str,
    #     'home.name': str,
    #     'home.abbreviation': str,
    #     'home.displayName': str,
    #     'home.shortDisplayName': str,
    #     'home.color': str,
    #     'home.alternateColor': str,
    #     'home.isActive': bool,
    #     'home.venue.id': int,
    #     'home.logo': str,
    #     'home.conferenceId': int,
    #     'home.score': int,
    #     'home.winner': bool,
    #     'away.id': int,
    #     'away.uid': str,
    #     'away.location': str,
    #     'away.name': str,
    #     'away.abbreviation': str,
    #     'away.displayName': str,
    #     'away.shortDisplayName': str,
    #     'away.color': str,
    #     'away.alternateColor': str,
    #     'away.isActive': bool,
    #     'away.venue.id': int,
    #     'away.logo': str,
    #     'away.conferenceId': int,
    #     'away.score': int,
    #     'away.winner': bool,
    #     'tournamentId': int
    # },errors='ignore')
    # print(ev.columns)
    return ev

def espn_mbb_calendar(season=None) -> pd.DataFrame:
    """espn_mbb_calendar - look up the men's college basketball calendar for a given season

    Args:
        season (int): Used to define different seasons. 2002 is the earliest available season.

    Returns:
        pd.DataFrame: Pandas dataframe containing schedule dates for the requested season.
    """
    if int(season) < 2002:
        raise SeasonNotFoundError("season cannot be less than 2002")
    url = "http://site.api.espn.com/apis/site/v2/sports/basketball/mens-college-basketball/scoreboard?dates={}".format(season)
    resp = download(url=url)
    txt = json.loads(resp)['leagues'][0]['calendar']
    datenum = list(map(lambda x: x[:10].replace("-",""),txt))
    date = list(map(lambda x: x[:10],txt))

    year = list(map(lambda x: x[:4],txt))
    month = list(map(lambda x: x[5:7],txt))
    day = list(map(lambda x: x[8:10],txt))

    data = {
        "season": season,
        "datetime" : txt,
        "date" : date,
        "year": year,
        "month": month,
        "day": day,
        "dateURL": datenum
    }
    df = pd.DataFrame(data)
    df['url']="http://site.api.espn.com/apis/site/v2/sports/basketball/mens-college-basketball/scoreboard?dates="
    df['url']= df['url'] + df['dateURL']
    return df


import pyarrow.parquet as pq
import pandas as pd
import json
from typing import List, Callable, Iterator, Union, Optional
from sportsdataverse.config import WNBA_BASE_URL, WNBA_TEAM_BOX_URL, WNBA_PLAYER_BOX_URL, WNBA_TEAM_SCHEDULE_URL
from sportsdataverse.errors import SeasonNotFoundError
from sportsdataverse.dl_utils import download

def load_wnba_pbp(seasons: List[int]) -> pd.DataFrame:
    """Load WNBA play by play data going back to 2002

    Example:
        `wnba_df = sportsdataverse.wnba.load_wnba_pbp(seasons=range(2002,2022))`

    Args:
        seasons (list): Used to define different seasons. 2002 is the earliest available season.

    Returns:
        pd.DataFrame: Pandas dataframe containing the
        play-by-plays available for the requested seasons.

    Raises:
        ValueError: If `season` is less than 2002.
    """
    data = pd.DataFrame()
    if type(seasons) is int:
        seasons = [seasons]
    for i in seasons:
        if int(i) < 2002:
            raise SeasonNotFoundError("season cannot be less than 2002")
        i_data = pd.read_parquet(WNBA_BASE_URL.format(season=i), engine='auto', columns=None)
        data = data.append(i_data)
    #Give each row a unique index
    data.reset_index(drop=True, inplace=True)
    return data

def load_wnba_team_boxscore(seasons: List[int]) -> pd.DataFrame:
    """Load WNBA team boxscore data

    Example:
        `wnba_df = sportsdataverse.wnba.load_wnba_team_boxscore(seasons=range(2002,2022))`

    Args:
        seasons (list): Used to define different seasons. 2002 is the earliest available season.

    Returns:
        pd.DataFrame: Pandas dataframe containing the
        team boxscores available for the requested seasons.

    Raises:
        ValueError: If `season` is less than 2002.
    """
    data = pd.DataFrame()
    if type(seasons) is int:
        seasons = [seasons]
    for i in seasons:
        if int(i) < 2002:
            raise ValueError("season cannot be less than 2002")
        i_data = pd.read_parquet(WNBA_TEAM_BOX_URL.format(season = i), engine='auto', columns=None)
        data = data.append(i_data)
    #Give each row a unique index
    data.reset_index(drop=True, inplace=True)

    return data

def load_wnba_player_boxscore(seasons: List[int]) -> pd.DataFrame:
    """Load WNBA player boxscore data

    Example:
        `wnba_df = sportsdataverse.wnba.load_wnba_player_boxscore(seasons=range(2002,2022))`

    Args:
        seasons (list): Used to define different seasons. 2002 is the earliest available season.

    Returns:
        pd.DataFrame: Pandas dataframe containing the
        player boxscores available for the requested seasons.

    Raises:
        ValueError: If `season` is less than 2002.
    """
    data = pd.DataFrame()
    if type(seasons) is int:
        seasons = [seasons]
    for i in seasons:
        if int(i) < 2002:
            raise ValueError("season cannot be less than 2002")
        i_data = pd.read_parquet(WNBA_PLAYER_BOX_URL.format(season = i), engine='auto', columns=None)
        data = data.append(i_data)
    #Give each row a unique index
    data.reset_index(drop=True, inplace=True)

    return data

def load_wnba_schedule(seasons: List[int]) -> pd.DataFrame:
    """Load WNBA schedule data

    Example:
        `wnba_df = sportsdataverse.wnba.load_wnba_schedule(seasons=range(2002,2022))`

    Args:
        seasons (list): Used to define different seasons. 2002 is the earliest available season.

    Returns:
        pd.DataFrame: Pandas dataframe containing the
        schedule for  the requested seasons.

    Raises:
        ValueError: If `season` is less than 2002.
    """
    data = pd.DataFrame()
    if type(seasons) is int:
        seasons = [seasons]
    for i in seasons:
        if int(i) < 2002:
            raise ValueError("season cannot be less than 2002")
        i_data = pd.read_parquet(WNBA_TEAM_SCHEDULE_URL.format(season = i), engine='auto', columns=None)
        data = data.append(i_data)
    #Give each row a unique index
    data.reset_index(drop=True, inplace=True)

    return data




import numpy as np
import time
import http.client
import urllib.request
import requests
import json
from urllib.error import URLError, HTTPError, ContentTooShortError
from datetime import datetime
from itertools import chain, starmap
class ESPNResponse:
    def __init__(self, response, status_code, url):
        self._response = response
        self._status_code = status_code
        self._url = url

    def get_response(self):
        return self._response

    def get_dict(self):
        return json.loads(self._response)

    def get_json(self):
        return json.dumps(self.get_dict())

    def valid_json(self):
        try:
            self.get_dict()
        except ValueError:
            return False
        return True

    def get_url(self):
        return self._url

class ESPNHTTP:

    espn_response = ESPNResponse

    base_url = None

    parameters = None

    headers = None

    def clean_contents(self, contents):
        return contents

    def send_api_request(self, endpoint, parameters, referer=None, headers=None, timeout=None, raise_exception_on_error=False):
        if not self.base_url:
            raise Exception('Cannot use send_api_request from _HTTP class.')
        base_url = self.base_url.format(endpoint=endpoint)
        endpoint = endpoint.lower()
        self.parameters = parameters

        if headers is None:
            request_headers = self.headers
        else:
            request_headers = headers

        if referer:
            request_headers['Referer'] = referer

        url = None
        status_code = None
        contents = None

        # Sort parameters by key... for some reason this matters for some requests...
        parameters = sorted(parameters.items(), key=lambda kv: kv[0])

        if not contents:
            response = requests.get(url=base_url, params=parameters, headers=request_headers, timeout=timeout)
            url = response.url
            status_code = response.status_code
            contents = response.text

        contents = self.clean_contents(contents)

        data = self.espn_response(response=contents, status_code=status_code, url=url)

        if raise_exception_on_error and not data.valid_json():
            raise Exception('InvalidResponse: Response is not in a valid JSON format.')

        return data

def download(url, params = {}, num_retries=10):
    try:
        html = urllib.request.urlopen(url).read()
    except (URLError, HTTPError, ContentTooShortError, http.client.HTTPException, http.client.IncompleteRead) as e:
        print('Download error:', url)
        html = None
        if num_retries > 0:
            if hasattr(e, 'code') and 500 <= e.code < 600:
                time.sleep(10)
                # recursively retry 5xx HTTP errors
                return download(url, num_retries - 1)
        if num_retries > 0:
            if e == http.client.IncompleteRead:
                time.sleep(10)
                return download(url, num_retries - 1)
        if num_retries == 0:
            print("Retry Limit Exceeded")
    return html

def flatten_json_iterative(dictionary, sep = '.', ind_start = 0):
    """Flattening a nested json file"""

    def unpack_one(parent_key, parent_value):
        """Unpack one level (only one) of nesting in json file"""
        # Unpacking one level
        if isinstance(parent_value, dict):
            for key, value in parent_value.items():
                t1 = parent_key + sep + key
                yield t1, value
        elif isinstance(parent_value, list):
            i = ind_start
            for value in parent_value:
                t2 = parent_key + sep +str(i)
                i += 1
                yield t2, value
        else:
            yield parent_key, parent_value
    # Continue iterating the unpack_one function until the terminating condition is satisfied
    while True:
        # Continue unpacking the json file until all values are atomic elements (aka neither a dictionary nor a list)
        dictionary = dict(chain.from_iterable(starmap(unpack_one, dictionary.items())))
        # Terminating condition: none of the values in the json file are a dictionary or a list
        if not any(isinstance(value, dict) for value in dictionary.values()) and \
        not any(isinstance(value, list) for value in dictionary.values()):
            break
    return dictionary

def key_check(obj, key, replacement = np.array([])):
    if key in obj.keys():
        obj_key = obj[key]
    else:
        obj_key = replacement
    return obj_key


import pyarrow.parquet as pq
import pandas as pd
import json
from typing import List, Callable, Iterator, Union, Optional
from sportsdataverse.config import NFL_BASE_URL, NFL_ROSTER_URL, NFL_TEAM_LOGO_URL, NFL_TEAM_SCHEDULE_URL, NFL_PLAYER_STATS_URL
from sportsdataverse.errors import SeasonNotFoundError
from sportsdataverse.dl_utils import download

def load_nfl_pbp(seasons: List[int]) -> pd.DataFrame:
    """Load NFL play by play data going back to 1999

    Example:
        `nfl_df = sportsdataverse.nfl.load_nfl_pbp(seasons=range(1999,2021))`

    Args:
        seasons (list): Used to define different seasons. 1999 is the earliest available season.

    Returns:
        pd.DataFrame: Pandas dataframe containing the play-by-plays available for the requested seasons.

    Raises:
        ValueError: If `season` is less than 1999.
    """
    data = pd.DataFrame()
    if type(seasons) is int:
        seasons = [seasons]
    for i in seasons:
        if int(i) < 1999:
            raise SeasonNotFoundError("season cannot be less than 1999")
        i_data = pd.read_parquet(NFL_BASE_URL.format(season=i), engine='auto', columns=None)
        data = data.append(i_data)
    #Give each row a unique index
    data.reset_index(drop=True, inplace=True)
    return data

def load_nfl_schedule(seasons: List[int]) -> pd.DataFrame:
    """Load NFL schedule data

    Example:
        `nfl_df = sportsdataverse.nfl.load_nfl_schedule(seasons=range(1999,2021))`

    Args:
        seasons (list): Used to define different seasons. 1999 is the earliest available season.

    Returns:
        pd.DataFrame: Pandas dataframe containing the schedule for the requested seasons.

    Raises:
        ValueError: If `season` is less than 1999.
    """
    data = pd.DataFrame()
    if type(seasons) is int:
        seasons = [seasons]
    for i in seasons:
        if int(i) < 1999:
            raise SeasonNotFoundError("season cannot be less than 1999")
        i_data = pd.read_parquet(NFL_TEAM_SCHEDULE_URL.format(season = i), engine='auto', columns=None)
        data = data.append(i_data)
    #Give each row a unique index
    data.reset_index(drop=True, inplace=True)

    return data

def load_nfl_player_stats() -> pd.DataFrame:
    """Load NFL player stats data

    Example:
        `nfl_df = sportsdataverse.nfl.load_nfl_player_stats()`

    Args:

    Returns:
        pd.DataFrame: Pandas dataframe containing player stats.
    """
    data = pd.DataFrame()
    i_data = pd.read_parquet(NFL_PLAYER_STATS_URL, engine='auto', columns=None)
    data = data.append(i_data)
    #Give each row a unique index
    data.reset_index(drop=True, inplace=True)

    return data

def load_nfl_rosters() -> pd.DataFrame:
    """Load NFL roster data for all seasons

    Example:
        `nfl_df = sportsdataverse.nfl.load_nfl_rosters(seasons=range(1999,2021))`

    Returns:
        pd.DataFrame: Pandas dataframe containing rosters available for the requested seasons.

    """
    data = pd.DataFrame()

    data = pd.read_csv(NFL_ROSTER_URL, compression='gzip', error_bad_lines=False, low_memory=False)
    #Give each row a unique index
    data.reset_index(drop=True, inplace=True)

    return data

def load_nfl_teams() -> pd.DataFrame:
    """Load NFL team ID information and logos

    Example:
        `nfl_df = sportsdataverse.nfl.load_nfl_teams()`

    Args:

    Returns:
        pd.DataFrame: Pandas dataframe containing teams available for the requested seasons.
    """
    df = pd.read_csv(NFL_TEAM_LOGO_URL, low_memory=False)
    return df


from typing import Dict
import pyarrow.parquet as pq
import pandas as pd
import numpy as np
import re
import json
from typing import List, Callable, Iterator, Union, Optional, Dict
from sportsdataverse.dl_utils import download, flatten_json_iterative, key_check

def espn_wnba_pbp(game_id: int, raw = False) -> Dict:
    """espn_wnba_pbp() - Pull the game by id. Data from API endpoints - `wnba/playbyplay`, `wnba/summary`

        Args:
            game_id (int): Unique game_id, can be obtained from wnba_schedule().

        Returns:
            Dict: Dictionary of game data with keys - "gameId", "plays", "winprobability", "boxscore", "header",
             "broadcasts", "videos", "playByPlaySource", "standings", "leaders", "seasonseries", "timeouts",
             "pickcenter", "againstTheSpread", "odds", "predictor", "espnWP", "gameInfo", "season"

        Example:
            `wnba_df = sportsdataverse.wnba.espn_wnba_pbp(game_id=401370395)`
    """
    # play by play
    pbp_txt = {}
    pbp_txt['timeouts'] = {}
    # summary endpoint for pickcenter array
    summary_url = "http://site.api.espn.com/apis/site/v2/sports/basketball/wnba/summary?event={}".format(game_id)
    summary_resp = download(summary_url)
    summary = json.loads(summary_resp)
    for k in ['plays', 'seasonseries', 'videos', 'broadcasts', 'pickcenter', 'againstTheSpread', 'odds', 'winprobability', 'teamInfo', 'espnWP', 'leaders']:
        pbp_txt[k]=key_check(obj=summary, key = k, replacement = np.array([]))
    for k in ['boxscore','format', 'gameInfo', 'predictor', 'article', 'header', 'season', 'standings']:
        pbp_txt[k] = key_check(obj=summary, key = k, replacement = {})
    for k in ['news','shop']:
        if k in pbp_txt.keys():
            del pbp_txt[k]
    incoming_keys_expected = ['boxscore', 'format', 'gameInfo', 'leaders', 'seasonseries', 'broadcasts',
                              'predictor', 'pickcenter', 'againstTheSpread', 'odds', 'winprobability',
                              'header', 'plays', 'article', 'videos', 'standings',
                              'teamInfo', 'espnWP', 'season', 'timeouts']
    if raw == True:
        # reorder keys in raw format, appending empty keys which are defined later to the end
        pbp_json = {}
        for k in incoming_keys_expected:
            if k in pbp_txt.keys():
                pbp_json[k] = pbp_txt[k]
            else:
                pbp_json[k] = {}
        return pbp_json

    pbp_json = helper_wnba_pbp(game_id, pbp_txt)
    return pbp_json

def helper_wnba_pbp(game_id, pbp_txt):
    gameSpread, homeFavorite, gameSpreadAvailable = helper_wnba_pickcenter(pbp_txt)
    pbp_txt['gameInfo'] = pbp_txt['header']['competitions'][0]
    pbp_txt['season'] = pbp_txt['header']['season']
    pbp_txt['playByPlaySource'] = pbp_txt['header']['competitions'][0]['playByPlaySource']
    # Home and Away identification variables
    homeTeamId = int(pbp_txt['header']['competitions'][0]['competitors'][0]['team']['id'])
    awayTeamId = int(pbp_txt['header']['competitions'][0]['competitors'][1]['team']['id'])
    homeTeamMascot = str(pbp_txt['header']['competitions'][0]['competitors'][0]['team']['name'])
    awayTeamMascot = str(pbp_txt['header']['competitions'][0]['competitors'][1]['team']['name'])
    homeTeamName = str(pbp_txt['header']['competitions'][0]['competitors'][0]['team']['location'])
    awayTeamName = str(pbp_txt['header']['competitions'][0]['competitors'][1]['team']['location'])
    homeTeamAbbrev = str(pbp_txt['header']['competitions'][0]['competitors'][0]['team']['abbreviation'])
    awayTeamAbbrev = str(pbp_txt['header']['competitions'][0]['competitors'][1]['team']['abbreviation'])
    homeTeamNameAlt = re.sub("Stat(.+)", "St", str(homeTeamName))
    awayTeamNameAlt = re.sub("Stat(.+)", "St", str(awayTeamName))

    if (pbp_txt['playByPlaySource'] != "none") & (len(pbp_txt['plays'])>1):
        helper_wnba_pbp_features(game_id, pbp_txt, gameSpread, homeFavorite, gameSpreadAvailable, homeTeamId, awayTeamId, homeTeamMascot, awayTeamMascot, homeTeamName, awayTeamName, homeTeamAbbrev, awayTeamAbbrev, homeTeamNameAlt, awayTeamNameAlt)
    else:
        pbp_txt['plays'] = pd.DataFrame()
    pbp_txt['plays'] = pbp_txt['plays'].replace({np.nan: None})
    pbp_json = {
        "gameId": game_id,
        "plays" : pbp_txt['plays'].to_dict(orient='records'),
        "winprobability" : np.array(pbp_txt['winprobability']).tolist(),
        "boxscore" : pbp_txt['boxscore'],
        "header" : pbp_txt['header'],
        "broadcasts" : np.array(pbp_txt['broadcasts']).tolist(),
        "videos" : np.array(pbp_txt['videos']).tolist(),
        "playByPlaySource": pbp_txt['playByPlaySource'],
        "standings" : pbp_txt['standings'],
        "leaders" : np.array(pbp_txt['leaders']).tolist(),
        "seasonseries" : np.array(pbp_txt['seasonseries']).tolist(),
        "timeouts" : pbp_txt['timeouts'],
        "pickcenter" : np.array(pbp_txt['pickcenter']).tolist(),
        "againstTheSpread" : np.array(pbp_txt['againstTheSpread']).tolist(),
        "odds" : np.array(pbp_txt['odds']).tolist(),
        "predictor" : pbp_txt['predictor'],
        "espnWP" : np.array(pbp_txt['espnWP']).tolist(),
        "gameInfo" : np.array(pbp_txt['gameInfo']).tolist(),
        "season" : np.array(pbp_txt['season']).tolist()
    }
    return pbp_json

def helper_wnba_pbp_features(game_id, pbp_txt, gameSpread, homeFavorite, gameSpreadAvailable, homeTeamId, awayTeamId, homeTeamMascot, awayTeamMascot, homeTeamName, awayTeamName, homeTeamAbbrev, awayTeamAbbrev, homeTeamNameAlt, awayTeamNameAlt):
    pbp_txt['plays_mod'] = []
    for play in pbp_txt['plays']:
        p = flatten_json_iterative(play)
        pbp_txt['plays_mod'].append(p)
    pbp_txt['plays'] = pd.json_normalize(pbp_txt,'plays_mod')
    pbp_txt['plays']['season'] = pbp_txt['season']['year']
    pbp_txt['plays']['seasonType'] = pbp_txt['season']['type']
    pbp_txt['plays']["awayTeamId"] = awayTeamId
    pbp_txt['plays']["awayTeamName"] = str(awayTeamName)
    pbp_txt['plays']["awayTeamMascot"] = str(awayTeamMascot)
    pbp_txt['plays']["awayTeamAbbrev"] = str(awayTeamAbbrev)
    pbp_txt['plays']["awayTeamNameAlt"] = str(awayTeamNameAlt)
    pbp_txt['plays']["homeTeamId"] = homeTeamId
    pbp_txt['plays']["homeTeamName"] = str(homeTeamName)
    pbp_txt['plays']["homeTeamMascot"] = str(homeTeamMascot)
    pbp_txt['plays']["homeTeamAbbrev"] = str(homeTeamAbbrev)
    pbp_txt['plays']["homeTeamNameAlt"] = str(homeTeamNameAlt)
        # Spread definition
    pbp_txt['plays']["homeTeamSpread"] = 2.5
    pbp_txt['plays']["gameSpread"] = abs(gameSpread)
    pbp_txt['plays']["homeTeamSpread"] = np.where(homeFavorite == True, abs(gameSpread), -1*abs(gameSpread))
    pbp_txt['homeTeamSpread'] = np.where(homeFavorite == True, abs(gameSpread), -1*abs(gameSpread))
    pbp_txt['plays']["homeFavorite"] = homeFavorite
    pbp_txt['plays']["gameSpread"] = gameSpread
    pbp_txt['plays']["gameSpreadAvailable"] = gameSpreadAvailable
    pbp_txt['plays'] = pbp_txt['plays'].to_dict(orient='records')
    pbp_txt['plays'] = pd.DataFrame(pbp_txt['plays'])
    pbp_txt['plays']['season'] = pbp_txt['header']['season']['year']
    pbp_txt['plays']['seasonType'] = pbp_txt['header']['season']['type']
    pbp_txt['plays']['game_id'] = int(game_id)
    pbp_txt['plays']["homeTeamId"] = homeTeamId
    pbp_txt['plays']["awayTeamId"] = awayTeamId
    pbp_txt['plays']["homeTeamName"] = str(homeTeamName)
    pbp_txt['plays']["awayTeamName"] = str(awayTeamName)
    pbp_txt['plays']["homeTeamMascot"] = str(homeTeamMascot)
    pbp_txt['plays']["awayTeamMascot"] = str(awayTeamMascot)
    pbp_txt['plays']["homeTeamAbbrev"] = str(homeTeamAbbrev)
    pbp_txt['plays']["awayTeamAbbrev"] = str(awayTeamAbbrev)
    pbp_txt['plays']["homeTeamNameAlt"] = str(homeTeamNameAlt)
    pbp_txt['plays']["awayTeamNameAlt"] = str(awayTeamNameAlt)
    pbp_txt['plays']['period.number'] = pbp_txt['plays']['period.number'].apply(lambda x: int(x))
    pbp_txt['plays']['qtr'] = pbp_txt['plays']['period.number'].apply(lambda x: int(x))


    pbp_txt['plays']["homeTeamSpread"] = 2.5
    pbp_txt['plays']["gameSpread"] = abs(gameSpread)
    pbp_txt['plays']["gameSpreadAvailable"] = gameSpreadAvailable
    pbp_txt['plays']["homeTeamSpread"] = np.where(homeFavorite == True, abs(gameSpread), -1*abs(gameSpread))
    pbp_txt['homeTeamSpread'] = np.where(homeFavorite == True, abs(gameSpread), -1*abs(gameSpread))
    pbp_txt['plays']["homeFavorite"] = homeFavorite
    pbp_txt['plays']["gameSpread"] = gameSpread
    pbp_txt['plays']["homeFavorite"] = homeFavorite

        #----- Time ---------------

    pbp_txt['plays']['clock.displayValue'] = np.select(
            [
                pbp_txt['plays']['clock.displayValue'].str.contains(":") == False
            ],
            [
                "0:" + pbp_txt['plays']['clock.displayValue'].apply(lambda x: str(x))
            ], default = pbp_txt['plays']['clock.displayValue']
        )
    pbp_txt['plays']['time'] = pbp_txt['plays']['clock.displayValue']
    pbp_txt['plays']['clock.mm'] = pbp_txt['plays']['clock.displayValue'].str.split(pat=':')
    pbp_txt['plays'][['clock.minutes','clock.seconds']] = pbp_txt['plays']['clock.mm'].to_list()
    pbp_txt['plays']['clock.minutes'] = pbp_txt['plays']['clock.minutes'].apply(lambda x: int(x))

    pbp_txt['plays']['clock.seconds'] = pbp_txt['plays']['clock.seconds'].apply(lambda x: float(x))
        # pbp_txt['plays']['clock.mm'] = pbp_txt['plays']['clock.displayValue'].apply(lambda x: datetime.strptime(str(x),'%M:%S'))
    pbp_txt['plays']['half'] = np.where(pbp_txt['plays']['qtr'] <= 2, "1","2")
    pbp_txt['plays']['game_half'] = np.where(pbp_txt['plays']['qtr'] <= 2, "1","2")
    pbp_txt['plays']['lag_qtr'] = pbp_txt['plays']['qtr'].shift(1)
    pbp_txt['plays']['lead_qtr'] = pbp_txt['plays']['qtr'].shift(-1)
    pbp_txt['plays']['lag_game_half'] = pbp_txt['plays']['game_half'].shift(1)
    pbp_txt['plays']['lead_game_half'] = pbp_txt['plays']['game_half'].shift(-1)
    pbp_txt['plays']['start.quarter_seconds_remaining'] = 60*pbp_txt['plays']['clock.minutes'].astype(int) + pbp_txt['plays']['clock.seconds'].astype(int)
    pbp_txt['plays']['start.half_seconds_remaining'] = np.where(
            pbp_txt['plays']['qtr'].isin([1,3]),
            600 + 60*pbp_txt['plays']['clock.minutes'].astype(int) + pbp_txt['plays']['clock.seconds'].astype(int),
            60*pbp_txt['plays']['clock.minutes'].astype(int) + pbp_txt['plays']['clock.seconds'].astype(int)
        )
    pbp_txt['plays']['start.game_seconds_remaining'] = np.select(
            [
                pbp_txt['plays']['qtr'] == 1,
                pbp_txt['plays']['qtr'] == 2,
                pbp_txt['plays']['qtr'] == 3,
                pbp_txt['plays']['qtr'] == 4
            ],
            [
                1800 + 60*pbp_txt['plays']['clock.minutes'].astype(int) + pbp_txt['plays']['clock.seconds'].astype(int),
                1200 + 60*pbp_txt['plays']['clock.minutes'].astype(int) + pbp_txt['plays']['clock.seconds'].astype(int),
                600 + 60*pbp_txt['plays']['clock.minutes'].astype(int) + pbp_txt['plays']['clock.seconds'].astype(int),
                60*pbp_txt['plays']['clock.minutes'].astype(int) + pbp_txt['plays']['clock.seconds'].astype(int)
            ], default = 60*pbp_txt['plays']['clock.minutes'].astype(int) + pbp_txt['plays']['clock.seconds'].astype(int)
        )
        # Pos Team - Start and End Id
    pbp_txt['plays']['game_play_number'] = np.arange(len(pbp_txt['plays']))+1
    pbp_txt['plays']['text'] = pbp_txt['plays']['text'].astype(str)
    pbp_txt['plays']['id'] = pbp_txt['plays']['id'].apply(lambda x: int(x))
    pbp_txt['plays']['end.quarter_seconds_remaining'] = pbp_txt['plays']['start.quarter_seconds_remaining'].shift(1)
    pbp_txt['plays']['end.half_seconds_remaining'] = pbp_txt['plays']['start.half_seconds_remaining'].shift(1)
    pbp_txt['plays']['end.game_seconds_remaining'] = pbp_txt['plays']['start.game_seconds_remaining'].shift(1)
    pbp_txt['plays']['end.quarter_seconds_remaining'] = np.select(
            [
                (pbp_txt['plays']['game_play_number'] == 1)|
                ((pbp_txt['plays']['qtr'] == 2) & (pbp_txt['plays']['lag_qtr'] == 1))|
                ((pbp_txt['plays']['qtr'] == 3) & (pbp_txt['plays']['lag_qtr'] == 2))|
                ((pbp_txt['plays']['qtr'] == 4) & (pbp_txt['plays']['lag_qtr'] == 3))
            ],
            [
                600
            ], default = pbp_txt['plays']['end.quarter_seconds_remaining']
        )
    pbp_txt['plays']['end.half_seconds_remaining'] = np.select(
            [
                (pbp_txt['plays']['game_play_number'] == 1)|
                ((pbp_txt['plays']['game_half'] == "2") & (pbp_txt['plays']['lag_game_half'] == "1"))
            ],
            [
                1200
            ], default = pbp_txt['plays']['end.half_seconds_remaining']
        )
    pbp_txt['plays']['end.game_seconds_remaining'] = np.select(
            [
                (pbp_txt['plays']['game_play_number'] == 1),
                ((pbp_txt['plays']['game_half'] == "2") & (pbp_txt['plays']['lag_game_half'] == "1"))
            ],
            [
                2400,
                1200
            ], default = pbp_txt['plays']['end.game_seconds_remaining']
        )

    pbp_txt['plays']['period'] = pbp_txt['plays']['qtr']

    del pbp_txt['plays']['clock.mm']

def helper_wnba_pickcenter(pbp_txt):
    if len(pbp_txt['pickcenter']) > 1:
        if 'spread' in pbp_txt['pickcenter'][1].keys():
            gameSpread =  pbp_txt['pickcenter'][1]['spread']
            homeFavorite = pbp_txt['pickcenter'][1]['homeTeamOdds']['favorite']
            gameSpreadAvailable = True
        else:
            gameSpread =  pbp_txt['pickcenter'][0]['spread']
            homeFavorite = pbp_txt['pickcenter'][0]['homeTeamOdds']['favorite']
            gameSpreadAvailable = True

    else:
        gameSpread = 2.5
        homeFavorite = True
        gameSpreadAvailable = False
    return gameSpread,homeFavorite,gameSpreadAvailable




import pandas as pd
import json
from sportsdataverse.dl_utils import download
from urllib.error import URLError, HTTPError, ContentTooShortError

def espn_wnba_teams() -> pd.DataFrame:
    """espn_wnba_teams - look up WNBA teams

    Returns:
        pd.DataFrame: Pandas dataframe containing teams for the requested league.
    """
    ev = pd.DataFrame()
    url = "http://site.api.espn.com/apis/site/v2/sports/basketball/wnba/teams?limit=1000"
    resp = download(url=url)
    if resp is not None:
        events_txt = json.loads(resp)

        teams = events_txt.get('sports')[0].get('leagues')[0].get('teams')
        del_keys = ['record', 'links']
        for team in teams:
            for k in del_keys:
                team.get('team').pop(k, None)
        teams = pd.json_normalize(teams)
    return teams




from typing import Dict
import pyarrow.parquet as pq
import pandas as pd
import numpy as np
import re
import json
from typing import List, Callable, Iterator, Union, Optional, Dict
from sportsdataverse.dl_utils import download, flatten_json_iterative, key_check


def nhl_api_pbp(game_id: int) -> Dict:
    """nhl_api_pbp() - Pull the game by id. Data from API endpoints - `nhl/playbyplay`, `nhl/summary`

    Args:
        game_id (int): Unique game_id, can be obtained from nhl_schedule().

    Returns:
        Dict: Dictionary of game data with keys - "gameId", "plays", "boxscore", "header", "broadcasts",
         "videos", "playByPlaySource", "standings", "leaders", "seasonseries", "pickcenter", "againstTheSpread",
         "odds", "onIce", "gameInfo", "season"

    Example:
        `nhl_df = sportsdataverse.nhl.nhl_api_pbp(game_id=2021020079)`
    """
    # play by play
    pbp_txt = {}
    # summary endpoint for pickcenter array
    summary_url = "https://statsapi.web.nhl.com/api/v1/game/{}/feed/live?site=en_nhl".format(game_id)
    summary_resp = download(summary_url)
    summary = json.loads(summary_resp)
    pbp_txt['datetime'] = summary.get("gameData").get("datetime")
    pbp_txt['game'] = summary.get("gameData").get("game")
    pbp_txt['players'] = summary.get("gameData").get("players")
    pbp_txt['status'] = summary.get("gameData").get("status")
    pbp_txt['teams'] = summary.get("gameData").get("teams")
    pbp_txt['venues'] = summary.get("gameData").get("venues")
    pbp_txt['gameId'] = summary.get("gameData").get("gamePk")
    pbp_txt['gameLink'] = summary.get("gameData").get("link")
    return pbp_txt


def nhl_api_schedule(start_date: str, end_date: str) -> Dict:
    """nhl_api_schedule() - Pull the game by id. Data from API endpoints - `nhl/schedule`

    Args:
        game_id (int): Unique game_id, can be obtained from nhl_schedule().

    Returns:
        Dict:

    Example:
        `nhl_sched_df = sportsdataverse.nhl.nhl_api_schedule(start_date=2021-10-23, end_date=2021-10-28)`
    """
    # play by play
    pbp_txt = {}
    # summary endpoint for pickcenter array
    summary_url = "https://statsapi.web.nhl.com/api/v1/schedule?site=en_nhl&startDate={}&endDate={}".format(start_date, end_date)
    summary_resp = download(summary_url)
    summary = json.loads(summary_resp)
    pbp_txt['dates'] = summary.get("dates")
    pbp_txt_games = pd.DataFrame()
    for date in pbp_txt['dates']:
        game = pd.json_normalize(date, record_path="games", meta=["date"])
        pbp_txt_games = pbp_txt_games.append(game, ignore_index=True)
    return pbp_txt_games